# redux-substore

Collected actions, reducer, epic and state for any async operation.

[![Build Status](https://travis-ci.org/jbeuckm/redux-substore.svg?branch=master)](https://travis-ci.org/jbeuckm/redux-substore)

## Install 

`npm install redux-substore --save`


## Run the demo!

`npm run demo`

The demo shows how `redux-substore` can manage two types of requests: one that succeeds and one that fails. 

## How to use it:

### Setup

Construct a new __Substore__ that will manage the `getMyData()` request function.

```
import Substore from 'redux-substore'

export default new Substore({
  prefix: 'MY_PREFIX',
  promiseFunction: args => getMyData(args),
  responseMap: response => response['keyWhereMyDataIs'],
})
```

The `prefix` will be used to label the action types for this substore. The function passed to `promiseFunction:` must return a promise that will resolve with the response or reject with an error. Alternatively, send `callbackFunction:` with a Node-style callback with signature `function(err, data)`. The `responseMap:` function will receive whatever the request function resolves or calls back with, and should use the response to generate the substore's state. `responseMap` should generate the initialState for the substore when it receives `null`.

### Integrate with Redux

Add the reducer and epic to your `combineReducers` and `combineEpics`:

```
import { combineReducers } from 'redux'

rootReducer = combineReducers({
	...
	myData: myDataSubstore.reducer
})
```
```
import { combineEpics } from 'redux-most'

rootEpic = combineEpics([
	...
	myDataSubstore.epic
])
```

### Manage Your Async Ops

Trigger the request by dispatching the result of `requestAction()`

```
store.dispatch(mySubstore.requestAction(requestParams))
```

Parameters passed to the `requestAction` will be passed on to the request function supplied as `promiseFunction:` or `callbackFunction:` when the __Substore__ was constructed. The action has type __MY\_PREFIX\_REQUEST__ and when it completes, __MY\_PREFIX\_REQUEST\_SUCCESS__ or __MY\_PREFIX\_REQUEST\_FAILURE__ will be dispatched accordingly.

Access the status and data of the substore by selecting the key you assigned in `combineReducers`:

`getState().myData.isLoading`

`getState().myData.error`

`getState().myData.data` *(default responseMap sets this key)*

### Clean up

If you need to remove the data in the __Substore__, dispatch `mySubstore.clearAction()`.
